/***************************************************************************
 *   Copyright (C) 2008 by Mikhail Zaslavskiy   *
 *   mikhail.zaslavskiy@ensmp.fr   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "algorithm_path_ca.h"
//gradient for qcc function, here gm_A*_d are the laplacian matrices,update current gradient value
void algorithm_path_ca::qcc_gradient_opt(gsl_matrix *gm_Ag_d,gsl_matrix *gm_Ah_d,gsl_vector* gv_P, gsl_vector* gv_grad,double dmult,gsl_matrix * gm_temp)
{
	//the only difference: we need to substitute adjacency matrix on the place of the laplacian
	gsl_matrix_scale(gm_Ah_d,-1);
	gsl_matrix_scale(gm_Ag_d,-1);
	for (int i=0;i<N;i++)
	{	gsl_matrix_set(gm_Ag_d,i,i,0);
		gsl_matrix_set(gm_Ah_d,i,i,0);
	};

	algorithm_path::qcc_gradient_opt(gm_Ag_d,gm_Ah_d,gv_P,gv_grad,dmult,gm_temp);

	gsl_vector* gv_ones = gsl_vector_alloc(N);
	gsl_vector* gv_res = gsl_vector_alloc(N);
	gsl_vector_set_all(gv_ones,1);
	gsl_blas_dgemv(CblasNoTrans,1,gm_Ag_d,gv_ones,0,gv_res);
	gsl_matrix_scale(gm_Ag_d,-1);
	for (int i=0;i<N;i++)
		gsl_matrix_set(gm_Ag_d,i,i,gsl_vector_get(gv_res,i));
	
	gsl_blas_dgemv(CblasNoTrans,1,gm_Ah_d,gv_ones,0,gv_res);
	gsl_matrix_scale(gm_Ah_d,-1);
	for (int i=0;i<N;i++)
		gsl_matrix_set(gm_Ah_d,i,i,gsl_vector_get(gv_res,i));
	
	gsl_vector_free(gv_res);
	gsl_vector_free(gv_ones);
}

//concave function value
double algorithm_path_ca::f_qcc(gsl_matrix *gm_Ag_d,gsl_matrix *gm_Ah_d,gsl_matrix* gm_Delta,gsl_matrix* gm_P,gsl_matrix * gm_temp,gsl_matrix *gm_temp2)
{
	//the only difference: we need to substitute adjacency matrix on the place of the laplacian
	gsl_matrix_scale(gm_Ah_d,-1);
	gsl_matrix_scale(gm_Ag_d,-1);
	for (int i=0;i<N;i++)
	{	gsl_matrix_set(gm_Ag_d,i,i,0);
		gsl_matrix_set(gm_Ah_d,i,i,0);
	};

	double dres=algorithm_path::f_qcc(gm_Ag_d,gm_Ah_d,gm_Delta,gm_P,gm_temp,gm_temp2);
	
	gsl_vector* gv_ones = gsl_vector_alloc(N);
	gsl_vector* gv_res = gsl_vector_alloc(N);
	gsl_vector_set_all(gv_ones,1);
	gsl_blas_dgemv(CblasNoTrans,1,gm_Ag_d,gv_ones,0,gv_res);
	gsl_matrix_scale(gm_Ag_d,-1);
	for (int i=0;i<N;i++)
		gsl_matrix_set(gm_Ag_d,i,i,gsl_vector_get(gv_res,i));
	
	gsl_blas_dgemv(CblasNoTrans,1,gm_Ah_d,gv_ones,0,gv_res);
	gsl_matrix_scale(gm_Ah_d,-1);
	for (int i=0;i<N;i++)
		gsl_matrix_set(gm_Ah_d,i,i,gsl_vector_get(gv_res,i));
	
	gsl_vector_free(gv_res);
	gsl_vector_free(gv_ones);
	return dres;
}
/*match_result algorithm_path_ca::match(graph& g, graph& h,gsl_matrix* gm_P_i, gsl_matrix* _gm_ldh,double dalpha_ldh)
{
	double t=1;
	return algorithm_path::match(g,h,gm_P_i,_gm_ldh,dalpha_ldh);
}
*/
